==> redpesk-cli_3.15.0.orig.tar.gz: redpesk-cli-3.15.0/cmd/deployments.go <==
/*
Copyright © 2021-2023 IoT.bzh Company
Contact: https://www.iot.bzh/licensing

This file is part of the redpesk-cli project, the redpesk command line tool.

$RP_BEGIN_LICENSE$
Commercial License Usage
 Licensees holding valid commercial IoT.bzh licenses may use this file in
 accordance with the commercial license agreement provided with the
 Software or, alternatively, in accordance with the terms contained in
 a written agreement between you and The IoT.bzh Company. For licensing terms
 and conditions see https://www.iot.bzh/terms-conditions. For further
 information use the contact form at https://www.iot.bzh/contact.

GNU General Public License Usage
 Alternatively, this file may be used under the terms of the GNU General
 Public license version 3. This license is as published by the Free Software
 Foundation and appearing in the file LICENSE.GPLv3 included in the packaging
 of this file. Please review the following information to ensure the GNU
 General Public License requirements will be met
 https://www.gnu.org/licenses/gpl-3.0.html.
$RP_END_LICENSE$
*/
package cmd

import (
	"encoding/json"
	"fmt"
	"text/tabwriter"

	"git.ovh.iot/redpesk/redpesk-infra/redpesk-cli/printer"
	"git.ovh.iot/redpesk/redpesk-infra/redpesk-cli/utils"
	"git.ovh.iot/redpesk/redpesk-infra/rp-webserver/actions/rpapiv1"
	"github.com/gofrs/uuid"
	"github.com/spf13/cobra"
)

func initDeploymentsCmd(rootCmd *cobra.Command) {

	// deploymentsCmd represents the deployments command
	var deploymentsCmd = &cobra.Command{
		Use:   deploymentsCommand,
		Short: deploymentsCmdShortDesc,
	}

	// listCmd represents the list command
	var listCmd = &cobra.Command{
		Use:   deploymentsListCmd,
		Short: deploymentsListCmdShortDesc,
		Long:  deploymentsListCmdLongDesc,
		RunE:  listDeployments,
	}

	// getCmd represents the get command
	var getCmd = &cobra.Command{
		Use:               deploymentsGetCmd,
		Short:             deploymentsGetCmdShortUsage,
		RunE:              getDeployment,
		ValidArgsFunction: getListDeploymentsAsTab,
		Annotations:       map[string]string{"__arg_need_bck_connect__": "TRUE"},
	}

	// Here you will define your flags and configuration settings.

	// Persistent Flags -----------------------------------------
	// ----------------------------------------------------------

	// Local Flags ----------------------------------------------
	listCmd.Flags().BoolP(mainVerboseFlag, mainVerboseFlagAlias, mainVerboseFlagDefault, mainVerboseFlagUsage)
	listCmd.Flags().StringP(deploymentsCmdPrjFlag, deploymentsCmdPrjFlagAlias, "", deploymentsCmdPrjFlagUsage)
	getCmd.Flags().BoolP(mainVerboseFlag, mainVerboseFlagAlias, mainVerboseFlagDefault, mainVerboseFlagUsage)
	// ----------------------------------------------------------

	// Add some flags as required -------------------------------
	// ----------------------------------------------------------

	// Add the group commands "deployments" as subcommand of rootCmd
	rootCmd.AddCommand(deploymentsCmd)
	// Add the command "list" as a subcommand of deploymentsCmd
	deploymentsCmd.AddCommand(listCmd)
	// Add the command "get" as a subcommand of deploymentsCmd
	deploymentsCmd.AddCommand(getCmd)
}

// --------------------------------------------------------------
// Completion functions -----------------------------------------
// --------------------------------------------------------------

// getListDeploymentsAsTab gets all the deployments in the Redpesk
// instance and return it as an array of strings
//
// :param cmd: Pointer to the cobra command received
// :type cmd: *cobra.Command
// :param args: Arguments passed by the user to the command
// :type args: []string
// :param toComplete: String passed by the user to complete
// :type toComplete: string
// :returns: []string, cobra.ShellCompDirective -- the tab result and the shell completion directive
func getListDeploymentsAsTab(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
	deploymentsList := []string{}

	// Get the list of every available deployments
	deployments := rpapiv1.RPDeployments{}
	if err := bckAPI.ListDeployments(&deployments, ""); err != nil {
		return deploymentsList, cobra.ShellCompDirectiveNoFileComp
	}

	// We got correctly the deployments list, fill the deploymentsList tab
	for _, deployment := range deployments.Deployments {
		deploymentsList = append(deploymentsList, deployment.ID.String())
	}

	return deploymentsList, cobra.ShellCompDirectiveNoFileComp
}

// --------------------------------------------------------------
// --------------------------------------------------------------

// --------------------------------------------------------------
// Command functions --------------------------------------------
// --------------------------------------------------------------

// listDeployments gets and prints the list of all the deployments
// that have been done using the redpesk factory
//
// :param cmd: Pointer to the cobra command received
// :type cmd: *cobra.Command
// :param args: Arguments passed by the user to the command
// :type args: []string
// :returns: error -- an error if something went bad
func listDeployments(cmd *cobra.Command, args []string) error {

	// Get the verbose flag
	verbose, err := cmd.Flags().GetBool(mainVerboseFlag)
	if err != nil {
		logFlagError(mainVerboseFlag)
		return err
	}

	// Get the "released-project" flag
	relProjectID, err := cmd.Flags().GetString(deploymentsCmdPrjFlag)
	if err != nil {
		logFlagError(deploymentsCmdPrjFlag)
		return err
	}

	// Get the corresponding released project UUID
	if relProjectID != "" {
		// Get the UUID for the project
		tmpUUID, err := getRelProjectUUID(relProjectID)
		if err != nil {
			return err
		}
		relProjectID = tmpUUID.String()
	}

	// Prepare the function needed by the printer to get Released Projects
	getDeploymentsCB := func(pageNb, perPage int) (interface{}, printer.RPPrinterPaging, error) {
		// Get deployments list
		deployments := rpapiv1.RPDeployments{}
		if err := bckAPI.ListDeploymentsByPage(&deployments, relProjectID, pageNb, perPage); err != nil {
			return deployments.Deployments, printer.RPPrinterPaging{}, err
		}
		// Get the paging interface from the structure retrieved
		pagingStruct, _ := printer.GetPagingFromInterface(&deployments)

		return deployments.Deployments, pagingStruct, nil
	}

	// Prepare the structure to manage paging
	pagingStruct := printer.RPPagingToolsStruct{
		Verbose:       verbose,
		VerboseLineNb: deploymentDisplayVerboseLightLineNb,
		GetObjCB:      getDeploymentsCB,
		PrintFancyCB:  displayDeployments,
	}

	return RPPrinter.InteractivelyPrintPages(&pagingStruct)
}

// getDeployment gets and prints the given deployment
//
// :param cmd: Pointer to the cobra command received
// :type cmd: *cobra.Command
// :param args: Arguments passed by the user to the command
// :type args: []string
// :returns: error -- an error if something went bad
func getDeployment(cmd *cobra.Command, args []string) error {

	// Get the verbose flag
	verbose, err := cmd.Flags().GetBool(mainVerboseFlag)
	if err != nil {
		logFlagError(mainVerboseFlag)
		return err
	}

	// Get the deployment ID wanted by the user
	dplID, err := getFirstArg(args, fmt.Sprintf(objectIDNotSetMsg, "deployment"), fmt.Sprintf(objectIDRequiredMsg, "deployment"))
	if err != nil {
		return err
	}

	// From the ID given by the user, get the deployment UUID
	dplUUID, err := getDeploymentUUID(dplID)
	if err != nil {
		return err
	}

	// Get the full deployment structure from the server
	deploymentStruct := rpapiv1.RPDeploymentFull{}
	if err = bckAPI.GetDeployment(dplUUID.String(), &deploymentStruct); err != nil {
		return err
	}

	// Print the result to the user
	err = RPPrinter.CustomPrintObject(deploymentStruct, verbose, displayOneDeployment)

	return err
}

// --------------------------------------------------------------
// --------------------------------------------------------------

// --------------------------------------------------------------
// Utilities functions ------------------------------------------
// --------------------------------------------------------------

// getDeploymentUUID check that the ID given by the user is an UUID and that it
// exists
//
// :param deploymentID: Identifier of the deployment given by the user
// :type deploymentID: string
// :returns: uuid.UUID, error -- the UUID of the deployment and an error if something went bad
func getDeploymentUUID(deploymentID string) (uuid.UUID, error) {
	var err error
	deploymentUUID := uuid.Nil

	// Get the list of every available deployments
	deployments := rpapiv1.RPDeployments{}
	if err := bckAPI.ListDeployments(&deployments, ""); err != nil {
		return deploymentUUID, err
	}

	// Check if the ID given is a UUID
	dplUUID, err := uuid.FromString(deploymentID)
	if err != nil {
		return deploymentUUID, fmt.Errorf(deploymentIdentifierNotUUID, deploymentID)
	}

	// Find the deployment that is interesting us
	deploymentFound := false
	for _, deployment := range deployments.Deployments {
		if utils.UUIDEqual(dplUUID, deployment.ID) {
			deploymentUUID = deployment.ID
			deploymentFound = true
			break
		}
	}

	if !deploymentFound {
		err = fmt.Errorf(deploymentIdentifierNotFound, deploymentID)
	} else {
		err = nil
	}

	return deploymentUUID, err
}

// --------------------------------------------------------------
// --------------------------------------------------------------

// --------------------------------------------------------------
// Display functions --------------------------------------------
// --------------------------------------------------------------

// displayDeployments is a private function allowing to display
// a slice of deployments in the console
//
// :param deployments: tab of deployments to display
// :type deployments: interface{}
// :param verbose: True if the user wants verbosity in the answer
// :type verbose: bool
// :param writer: Writer to use to print the information
// :type writer: *tabwriter.Writer
// :returns: error -- an error if something went bad
func displayDeployments(deployments interface{}, verbose bool, writer *tabwriter.Writer) error {
	// Convert to the right type
	deploys := deployments.([]rpapiv1.RPDeployment)

	// Display result
	for index, dpl := range deploys {
		if verbose && (index > 0) {
			fmt.Fprintln(writer)
		} else if !verbose && index == 0 {
			manageHeaderPrint(deploymentsListAnsHeader, writer)
		}
		// Print this project
		_printOneDeployment(dpl, verbose, false, writer)
	}

	return nil
}

// displayOneDeployment is a private function allowing to display
// one deployment in the console
//
// :param deployment: Deployment to display
// :type deployment: interface{}
// :param verbose: True if the user wants verbosity in the answer
// :type verbose: bool
// :returns: error -- an error if something went bad
func displayOneDeployment(deployment interface{}, verbose bool, writer *tabwriter.Writer) error {
	// Print the project
	return _printOneDeployment(deployment, verbose, true, writer)
}

// _printOneDeployment display one deployment in console
//
// :param deployment: Deployment to print
// :type deployment: interface{}
// :param verbose: Tell if the print needs to be verbose or not
// :type verbose: bool
// :param isAlone: Tell if the object is alone or not
// :type isAlone: bool
// :param writer: Writer to use to print the information
// :type writer: *tabwriter.Writer
// :returns: error -- an error if something went bad
func _printOneDeployment(deploymentInterface interface{}, verbose, isAlone bool, writer *tabwriter.Writer) error {
	deplToPrint := rpapiv1.RPDeploymentFull{}

	// Cast to the full type in all case but use "full" parameters as indicated by the user
	// We have to pass through the Marshal/Unmarshal tricks to not have problem during conversion from
	// RPDeployment -> RPDeploymentFull at runtime
	dataBytes, _ := json.Marshal(deploymentInterface)
	json.Unmarshal(dataBytes, &deplToPrint)

	// Define the name to print
	nameToPrint := mainNotSetMsg
	if deplToPrint.Name != "" {
		nameToPrint = deplToPrint.Name
	}

	if verbose {
		fmt.Fprintln(writer, deploymentsListAnsLine0, deplToPrint.ID.String())
		fmt.Fprintln(writer, deploymentsListAnsLine8, deplToPrint.ProjectDeployed.ProjectSlug)
		fmt.Fprintln(writer, deploymentsListAnsLine1, nameToPrint)
		fmt.Fprintln(writer, deploymentsListAnsLine2, deplToPrint.CreatedAt.String())
		fmt.Fprintln(writer, deploymentsListAnsLine3, deplToPrint.UpdatedAt.String())
		fmt.Fprintln(writer, deploymentsListAnsLine4, string(deplToPrint.Status))
		fmt.Fprintln(writer, deploymentsListAnsLine5, deplToPrint.CreatedTS.String())
		fmt.Fprintln(writer, deploymentsListAnsLine6, deplToPrint.FinishedTS.String())
		fmt.Fprintln(writer, deploymentsListAnsLine7, deplToPrint.UserLauncherPseudo)

		if deplToPrint.FullData {
			fmt.Fprintln(writer)
			fmt.Fprintln(writer, deploymentsListAnsBoardStatusTag)
			for _, deplBoard := range deplToPrint.DeploymentsPerBoard {
				fmt.Fprintln(writer, (deplBoard.Board.Name + ":"), string(deplBoard.Status))
				if deplBoard.Logs != "" {
					fmt.Fprintln(writer, "  ", "Logs:", deplBoard.Logs)
				}
			}

			fmt.Fprintln(writer)
			fmt.Fprintln(writer, deploymentsListAnsAppsRelTag)
			for index, deplApp := range deplToPrint.Apps {
				if index > 0 {
					fmt.Fprintln(writer)
				}
				fmt.Fprintln(writer, deploymentsListAnsLine9, deplApp.PackageName)
				fmt.Fprintln(writer, deploymentsListAnsLine10, deplApp.Branch)
				fmt.Fprintln(writer, deploymentsListAnsLine11, deplApp.ShaGitPkg)
				fmt.Fprintln(writer, deploymentsListAnsLine12, deplApp.BuildID)
			}
		}

	} else {
		if isAlone {
			fmt.Fprintln(writer, deploymentsListAnsHeader)
		}
		fmt.Fprintln(writer, deplToPrint.ID.String(), "\t", deplToPrint.ProjectDeployed.ProjectSlug, "\t", string(deplToPrint.Status), "\t", deplToPrint.CreatedTS.String(), "\t", nameToPrint)
	}

	return nil
}

// --------------------------------------------------------------
// --------------------------------------------------------------

==> redpesk-cli_3.18.3.orig.tar.gz: redpesk-cli-3.18.3/cmd/deployments.go <==
/*
Copyright © 2021-2023 IoT.bzh Company
Contact: https://www.iot.bzh/licensing

This file is part of the redpesk-cli project, the redpesk command line tool.

$RP_BEGIN_LICENSE$
Commercial License Usage

	Licensees holding valid commercial IoT.bzh licenses may use this file in
	accordance with the commercial license agreement provided with the
	Software or, alternatively, in accordance with the terms contained in
	a written agreement between you and The IoT.bzh Company. For licensing terms
	and conditions see https://www.iot.bzh/terms-conditions. For further
	information use the contact form at https://www.iot.bzh/contact.

GNU General Public License Usage

	Alternatively, this file may be used under the terms of the GNU General
	Public license version 3. This license is as published by the Free Software
	Foundation and appearing in the file LICENSE.GPLv3 included in the packaging
	of this file. Please review the following information to ensure the GNU
	General Public License requirements will be met
	https://www.gnu.org/licenses/gpl-3.0.html.

$RP_END_LICENSE$
*/
package cmd

import (
	"encoding/json"
	"fmt"
	"text/tabwriter"

	"git.ovh.iot/redpesk/redpesk-infra/redpesk-cli/printer"
	"git.ovh.iot/redpesk/redpesk-infra/redpesk-cli/utils"
	"git.ovh.iot/redpesk/redpesk-infra/rp-webserver/actions/rpapiv1"
	"github.com/gofrs/uuid"
	"github.com/spf13/cobra"
	"github.com/spf13/viper"
)

func initDeploymentsCmd(rootCmd *cobra.Command) {

	// deploymentsCmd represents the deployments command
	var deploymentsCmd = &cobra.Command{
		Use:   deploymentsCommand,
		Short: deploymentsCmdShortDesc,
	}

	// listCmd represents the list command
	var listCmd = &cobra.Command{
		Use:   deploymentsListCmd,
		Short: deploymentsListCmdShortDesc,
		Long:  deploymentsListCmdLongDesc,
		RunE:  listDeployments,
	}

	// getCmd represents the get command
	var getCmd = &cobra.Command{
		Use:               deploymentsGetCmd,
		Short:             deploymentsGetCmdShortUsage,
		RunE:              getDeployment,
		ValidArgsFunction: getListDeploymentsAsTab,
		Annotations:       map[string]string{"__arg_need_bck_connect__": "TRUE"},
	}

	// Here you will define your flags and configuration settings.

	// Persistent Flags -----------------------------------------
	// ----------------------------------------------------------

	// Local Flags ----------------------------------------------
	listCmd.Flags().BoolP(mainVerboseFlag, mainVerboseFlagAlias, mainVerboseFlagDefault, mainVerboseFlagUsage)
	listCmd.Flags().StringP(deploymentsCmdPrjFlag, deploymentsCmdPrjFlagAlias, "", deploymentsCmdPrjFlagUsage)
	getCmd.Flags().BoolP(mainVerboseFlag, mainVerboseFlagAlias, mainVerboseFlagDefault, mainVerboseFlagUsage)
	// ----------------------------------------------------------

	// Add some flags as required -------------------------------
	// ----------------------------------------------------------

	// Add the group commands "deployments" as subcommand of rootCmd
	rootCmd.AddCommand(deploymentsCmd)
	// Add the command "list" as a subcommand of deploymentsCmd
	deploymentsCmd.AddCommand(listCmd)
	// Add the command "get" as a subcommand of deploymentsCmd
	deploymentsCmd.AddCommand(getCmd)
}

// --------------------------------------------------------------
// Completion functions -----------------------------------------
// --------------------------------------------------------------

// getListDeploymentsAsTab gets all the deployments in the Redpesk
// instance and return it as an array of strings
//
// :param cmd: Pointer to the cobra command received
// :type cmd: *cobra.Command
// :param args: Arguments passed by the user to the command
// :type args: []string
// :param toComplete: String passed by the user to complete
// :type toComplete: string
// :returns: []string, cobra.ShellCompDirective -- the tab result and the shell completion directive
func getListDeploymentsAsTab(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
	deploymentsList := []string{}

	// Get the list of every available deployments
	deployments := rpapiv1.RPDeployments{}
	if err := bckAPI.ListDeployments(&deployments, ""); err != nil {
		return deploymentsList, cobra.ShellCompDirectiveNoFileComp
	}

	// We got correctly the deployments list, fill the deploymentsList tab
	for _, deployment := range deployments.Deployments {
		deploymentsList = append(deploymentsList, deployment.ID.String())
	}

	return deploymentsList, cobra.ShellCompDirectiveNoFileComp
}

// --------------------------------------------------------------
// --------------------------------------------------------------

// --------------------------------------------------------------
// Command functions --------------------------------------------
// --------------------------------------------------------------

// listDeployments gets and prints the list of all the deployments
// that have been done using the redpesk factory
//
// :param cmd: Pointer to the cobra command received
// :type cmd: *cobra.Command
// :param args: Arguments passed by the user to the command
// :type args: []string
// :returns: error -- an error if something went bad
func listDeployments(cmd *cobra.Command, args []string) error {

	// Get the verbose flag
	verbose, err := cmd.Flags().GetBool(mainVerboseFlag)
	if err != nil {
		logFlagError(mainVerboseFlag)
		return err
	}

	// Get the "released-project" flag
	relProjectID, err := cmd.Flags().GetString(deploymentsCmdPrjFlag)
	if err != nil {
		logFlagError(deploymentsCmdPrjFlag)
		return err
	}

	// Get the corresponding released project UUID
	if relProjectID != "" {
		relPrj, err := getRelProjectFromIdentifier(relProjectID, false)
		if err != nil {
			return err
		}
		relProjectID = relPrj.ID.String()
	}

	// Prepare the function needed by the printer to get Released Projects
	getDeploymentsCB := func(pageNb, perPage int) (interface{}, printer.RPPrinterPaging, error) {
		// Get deployments list
		deployments := rpapiv1.RPDeployments{}
		if err := bckAPI.ListDeploymentsByPage(&deployments, relProjectID, pageNb, perPage); err != nil {
			return deployments.Deployments, printer.RPPrinterPaging{}, err
		}
		// Get the paging interface from the structure retrieved
		pagingStruct, _ := printer.GetPagingFromInterface(&deployments)

		return deployments.Deployments, pagingStruct, nil
	}

	// Prepare the structure to manage paging
	pagingStruct := printer.RPPagingToolsStruct{
		Verbose:       verbose,
		VerboseLineNb: deploymentDisplayVerboseLightLineNb,
		GetObjCB:      getDeploymentsCB,
		PrintFancyCB:  displayDeployments,
	}

	return RPPrinter.InteractivelyPrintPages(&pagingStruct, viper.GetBool(NoHeaderFlag))
}

// getDeployment gets and prints the given deployment
//
// :param cmd: Pointer to the cobra command received
// :type cmd: *cobra.Command
// :param args: Arguments passed by the user to the command
// :type args: []string
// :returns: error -- an error if something went bad
func getDeployment(cmd *cobra.Command, args []string) error {

	// Get the verbose flag
	verbose, err := cmd.Flags().GetBool(mainVerboseFlag)
	if err != nil {
		logFlagError(mainVerboseFlag)
		return err
	}

	// Get the deployment ID wanted by the user
	dplID, err := getFirstArg(args, fmt.Sprintf(objectIDNotSetMsg, "deployment"), fmt.Sprintf(objectIDRequiredMsg, "deployment"))
	if err != nil {
		return err
	}

	// From the ID given by the user, get the deployment UUID
	dplUUID, err := getDeploymentUUID(dplID)
	if err != nil {
		return err
	}

	// Get the full deployment structure from the server
	deploymentStruct := rpapiv1.RPDeploymentFull{}
	if err = bckAPI.GetDeployment(dplUUID.String(), &deploymentStruct); err != nil {
		return err
	}

	// Print the result to the user
	err = RPPrinter.CustomPrintObject(deploymentStruct, verbose, displayOneDeployment)

	return err
}

// --------------------------------------------------------------
// --------------------------------------------------------------

// --------------------------------------------------------------
// Utilities functions ------------------------------------------
// --------------------------------------------------------------

// getDeploymentUUID check that the ID given by the user is an UUID and that it
// exists
//
// :param deploymentID: Identifier of the deployment given by the user
// :type deploymentID: string
// :returns: uuid.UUID, error -- the UUID of the deployment and an error if something went bad
func getDeploymentUUID(deploymentID string) (uuid.UUID, error) {
	var err error
	deploymentUUID := uuid.Nil

	// Get the list of every available deployments
	deployments := rpapiv1.RPDeployments{}
	if err := bckAPI.ListDeployments(&deployments, ""); err != nil {
		return deploymentUUID, err
	}

	// Check if the ID given is a UUID
	dplUUID, err := uuid.FromString(deploymentID)
	if err != nil {
		return deploymentUUID, fmt.Errorf(deploymentIdentifierNotUUID, deploymentID)
	}

	// Find the deployment that is interesting us
	deploymentFound := false
	for _, deployment := range deployments.Deployments {
		if utils.UUIDEqual(dplUUID, deployment.ID) {
			deploymentUUID = deployment.ID
			deploymentFound = true
			break
		}
	}

	if !deploymentFound {
		err = fmt.Errorf(deploymentIdentifierNotFound, deploymentID)
	} else {
		err = nil
	}

	return deploymentUUID, err
}

// --------------------------------------------------------------
// --------------------------------------------------------------

// --------------------------------------------------------------
// Display functions --------------------------------------------
// --------------------------------------------------------------

// displayDeployments is a private function allowing to display
// a slice of deployments in the console
//
// :param deployments: tab of deployments to display
// :type deployments: interface{}
// :param verbose: True if the user wants verbosity in the answer
// :type verbose: bool
// :param writer: Writer to use to print the information
// :type writer: *tabwriter.Writer
// :returns: error -- an error if something went bad
func displayDeployments(deployments interface{}, verbose bool, writer *tabwriter.Writer) error {
	// Convert to the right type
	deploys := deployments.([]rpapiv1.RPDeployment)

	// Display result
	for index, dpl := range deploys {
		if verbose && (index > 0) {
			fmt.Fprintln(writer)
		} else if !verbose && index == 0 {
			manageHeaderPrint(deploymentsListAnsHeader, writer)
		}
		// Print this project
		_printOneDeployment(dpl, verbose, false, writer)
	}

	return nil
}

// displayOneDeployment is a private function allowing to display
// one deployment in the console
//
// :param deployment: Deployment to display
// :type deployment: interface{}
// :param verbose: True if the user wants verbosity in the answer
// :type verbose: bool
// :returns: error -- an error if something went bad
func displayOneDeployment(deployment interface{}, verbose bool, writer *tabwriter.Writer) error {
	// Print the project
	return _printOneDeployment(deployment, verbose, true, writer)
}

// _printOneDeployment display one deployment in console
//
// :param deployment: Deployment to print
// :type deployment: interface{}
// :param verbose: Tell if the print needs to be verbose or not
// :type verbose: bool
// :param isAlone: Tell if the object is alone or not
// :type isAlone: bool
// :param writer: Writer to use to print the information
// :type writer: *tabwriter.Writer
// :returns: error -- an error if something went bad
func _printOneDeployment(deploymentInterface interface{}, verbose, isAlone bool, writer *tabwriter.Writer) error {
	deplToPrint := rpapiv1.RPDeploymentFull{}

	// Cast to the full type in all case but use "full" parameters as indicated by the user
	// We have to pass through the Marshal/Unmarshal tricks to not have problem during conversion from
	// RPDeployment -> RPDeploymentFull at runtime
	dataBytes, _ := json.Marshal(deploymentInterface)
	json.Unmarshal(dataBytes, &deplToPrint)

	// Define the name to print
	nameToPrint := mainNotSetMsg
	if deplToPrint.Name != "" {
		nameToPrint = deplToPrint.Name
	}

	if verbose {
		fmt.Fprintln(writer, deploymentsListAnsLine0, deplToPrint.ID.String())
		fmt.Fprintln(writer, deploymentsListAnsLine8, deplToPrint.ProjectDeployed.ProjectSlug)
		fmt.Fprintln(writer, deploymentsListAnsLine1, nameToPrint)
		fmt.Fprintln(writer, deploymentsListAnsLine2, deplToPrint.CreatedAt.String())
		fmt.Fprintln(writer, deploymentsListAnsLine3, deplToPrint.UpdatedAt.String())
		fmt.Fprintln(writer, deploymentsListAnsLine4, string(deplToPrint.Status))
		fmt.Fprintln(writer, deploymentsListAnsLine5, deplToPrint.CreatedTS.String())
		fmt.Fprintln(writer, deploymentsListAnsLine6, deplToPrint.FinishedTS.String())
		fmt.Fprintln(writer, deploymentsListAnsLine7, deplToPrint.UserLauncherPseudo)

		if deplToPrint.FullData {
			fmt.Fprintln(writer)
			fmt.Fprintln(writer, deploymentsListAnsBoardStatusTag)
			for _, deplBoard := range deplToPrint.DeploymentsPerBoard {
				fmt.Fprintln(writer, (deplBoard.Board.Name + ":"), string(deplBoard.Status))
				if deplBoard.Logs != "" {
					fmt.Fprintln(writer, "  ", "Logs:", deplBoard.Logs)
				}
			}

			fmt.Fprintln(writer)
			fmt.Fprintln(writer, deploymentsListAnsAppsRelTag)
			for index, deplApp := range deplToPrint.Apps {
				if index > 0 {
					fmt.Fprintln(writer)
				}
				fmt.Fprintln(writer, deploymentsListAnsLine9, deplApp.PackageName)
				fmt.Fprintln(writer, deploymentsListAnsLine10, deplApp.Branch)
				fmt.Fprintln(writer, deploymentsListAnsLine11, deplApp.ShaGitPkg)
				fmt.Fprintln(writer, deploymentsListAnsLine12, deplApp.BuildID)
			}
		}

	} else {
		if isAlone {
			fmt.Fprintln(writer, deploymentsListAnsHeader)
		}
		fmt.Fprintln(writer, deplToPrint.ID.String(), "\t", deplToPrint.ProjectDeployed.ProjectSlug, "\t", string(deplToPrint.Status), "\t", deplToPrint.CreatedTS.String(), "\t", nameToPrint)
	}

	return nil
}

// --------------------------------------------------------------
// --------------------------------------------------------------
